@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -35,12 +35,16 @@ GdkPixbuf *eel_create_colorized_pixbuf (GdkPixbuf *src,
 					int green_value,
 					int blue_value);
 
+char *     rb_utf_friendly_time        (time_t date);
+
 #ifdef HAVE_COLLATE_KEY_FILENAME
 #define rb_utf8_collate_key_for_filename g_utf8_collate_key_for_filename
 #else
 gchar*	rb_utf8_collate_key_for_filename (const gchar *str, gssize len);
 #endif
 
+char *     rb_make_valid_utf8 (const char *name, char substitute);
+
 G_END_DECLS
 
 #endif /* __RB_CUT_AND_PASTE_CODE_H */
