@@ -665,6 +665,13 @@ _rb_uri_recurse_data_free (RBUriHandleRecursivelyData *data)
 	g_free (data);
 }
 
+static gboolean
+_gnomevfs_info_unreadable (GnomeVFSFileInfo *info)
+{
+	return (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
+		!(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE);
+}
+
 static gboolean
 rb_uri_handle_recursively_cb (const gchar *rel_path,
 			      GnomeVFSFileInfo *info,
@@ -673,6 +680,7 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 			      gboolean *recurse)
 {
 	char *path, *escaped_rel_path;
+	char *sep;
 	gboolean dir;
 
 	dir = (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
@@ -681,9 +689,10 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 		return TRUE;
 
 	/* skip hidden and unreadable files and directories */
-	if (g_str_has_prefix (rel_path, ".") ||
-	    ((info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
-	    !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE))) {
+	sep = strrchr (rel_path, G_DIR_SEPARATOR);
+	if ((sep != NULL && g_str_has_prefix (sep + 1, ".")) ||
+	    (sep == NULL && g_str_has_prefix (rel_path, ".")) ||
+	    _gnomevfs_info_unreadable (info)) {
 		*recurse = FALSE;
 		return TRUE;
 	}
@@ -868,6 +877,8 @@ rb_canonicalise_uri (const char *uri)
 		result = gnome_vfs_make_path_name_canonical (uri);
 		tmp = gnome_vfs_escape_path_string (result);
 		g_free (result);
+		if (tmp == NULL)
+			return NULL;
 		tmp = escape_extra_gnome_vfs_chars (tmp);
 		result = g_strconcat ("file://", tmp, NULL);
 		g_free (tmp);
@@ -877,6 +888,8 @@ rb_canonicalise_uri (const char *uri)
 		tmp1  = gnome_vfs_unescape_string (uri + 7, NULL);  /* ignore "file://" */
 		tmp2 = gnome_vfs_escape_path_string (tmp1);
 		g_free (tmp1);
+		if (tmp2 == NULL)
+			return NULL;
 		tmp2 = escape_extra_gnome_vfs_chars (tmp2);
 		result = g_strconcat ("file://", tmp2, NULL); /* re-add scheme */
 		g_free (tmp2);
