@@ -29,26 +29,8 @@ ART_MISSING_ICON = 'rhythmbox-missing-artwork'
 WORKING_DELAY = 500
 THROBBER_RATE = 10
 THROBBER = 'gnome-spinner'
-
-def pixbuf_aspect (pixbuf):
-	if pixbuf is None: return None
-	return float (pixbuf.props.height) / pixbuf.props.width
-
-def weighted_avg (a, b, weight):
-	if a is None and b is None: return 1
-	elif a is None: return b
-	elif b is None: return a
-	else: return a * (1 - weight) + b * weight
-
-def merge_height (old_aspect, new_aspect, reserve_aspect, step, width):
-	if old_aspect is None:
-		if new_aspect is None:
-			return width
-		else:
-			return int (new_aspect * width)
-	elif new_aspect is None:
-		new_aspect = reserve_aspect
-	return int (width * weighted_avg (old_aspect, new_aspect, step))
+ASPECT_RATIO_MIN = 0.9
+ASPECT_RATIO_MAX = 1.1
 
 def merge_pixbufs (old_pb, new_pb, reserve_pb, step, width, height, mode=gtk.gdk.INTERP_BILINEAR):
 	if width <= 1 and height <= 1:
@@ -70,14 +52,25 @@ def merge_pixbufs (old_pb, new_pb, reserve_pb, step, width, height, mode=gtk.gdk
 	new_pb.composite (ret, 0, 0, width, height, 0, 0, sw, sh, mode, alpha)
 	return ret
 
-def merge_with_background (pixbuf, bgcolor):
-	if pixbuf is None or not pixbuf.get_has_alpha ():
+def merge_with_background (pixbuf, bgcolor, pad_if_not_near_square):
+	if pixbuf is None:
 		return pixbuf
+	has_alpha = pixbuf.get_has_alpha ()
 	width, height = pixbuf.props.width, pixbuf.props.height
-	bg = ((bgcolor.red & 0xff00) << 16) | ((bgcolor.green & 0xff00) << 8) | (bgcolor.blue & 0xff00) | 0xff
-	ret = gtk.gdk.Pixbuf (gtk.gdk.COLORSPACE_RGB, False, 8, width, height)
-	ret.fill (bg)
-	pixbuf.composite (ret, 0, 0, width, height, 0, 0, 1.0, 1.0, gtk.gdk.INTERP_NEAREST, 255)
+	if pad_if_not_near_square and (height < width * ASPECT_RATIO_MIN or
+				       height > width * ASPECT_RATIO_MAX):
+		rw, rh = max (width, height), max (width, height)
+		left, top = (rw - width) // 2, (rh - height) // 2
+	else:
+		if not has_alpha:
+			return pixbuf
+		rw, rh, left, top = width, height, 0, 0
+	ret = gtk.gdk.Pixbuf (gtk.gdk.COLORSPACE_RGB, False, 8, rw, rh)
+	ret.fill (((bgcolor.red & 0xff00) << 16) | ((bgcolor.green & 0xff00) << 8) | (bgcolor.blue & 0xff00) | 0xff)
+	if has_alpha:
+		pixbuf.composite (ret, left, top, width, height, left, top, 1.0, 1.0, gtk.gdk.INTERP_NEAREST, 255)
+	else:
+		pixbuf.copy_area (0, 0, width, height, ret, left, top)
 	return ret
 
 class FadingImage (gtk.Misc):
@@ -138,7 +131,7 @@ class FadingImage (gtk.Misc):
 			except Exception, e:
 				warn ("Missing artwork icon not found: %s" % e, Warning)
 				return
-		self.missing_pixbuf = merge_with_background (missing_pixbuf, self.style.bg[gtk.STATE_NORMAL])
+		self.missing_pixbuf = merge_with_background (missing_pixbuf, self.style.bg[gtk.STATE_NORMAL], False)
 
 	def do_size_allocate (self, allocation):
 		self.allocation = allocation
@@ -159,8 +152,7 @@ class FadingImage (gtk.Misc):
 		return False
 
 	def size_request (self, widget, requisition):
-		requisition.width = -1
-		requisition.height = merge_height (pixbuf_aspect (self.old_pixbuf), pixbuf_aspect (self.new_pixbuf), 1.0, self.fade_step, self.size)
+		requisition.width, requisition.height = -1, self.size
 
 	def expose (self, widget, event):
 		if not self.ensure_merged_pixbuf ():
@@ -223,7 +215,7 @@ class FadingImage (gtk.Misc):
 			self.old_pixbuf = self.render_overlay ()
 		else:
 			self.old_pixbuf = None	# don't fade
-		self.new_pixbuf = merge_with_background (pixbuf, self.style.bg[gtk.STATE_NORMAL])
+		self.new_pixbuf = merge_with_background (pixbuf, self.style.bg[gtk.STATE_NORMAL], True)
 		self.merged_pixbuf = None
 		self.fade_step = 0.0
 		self.anim = None
@@ -241,6 +233,20 @@ class FadingImage (gtk.Misc):
 gobject.type_register (FadingImage)
 
 
+class ArtDisplayWidget (FadingImage):
+	def __init__ (self, missing_image):
+		super (ArtDisplayWidget, self).__init__ (missing_image)
+		self.set_padding (0, 5)
+		self.current_entry, self.working = None, False
+		self.current_pixbuf, self.current_uri = None, None
+	
+	def set (self, entry, pixbuf, uri, working):
+		self.current_entry = entry
+		self.current_pixbuf = pixbuf
+		self.current_uri = uri
+		self.set_current_art (pixbuf, working)
+
+
 class ArtDisplayPlugin (rb.Plugin):
 	def __init__ (self):
 		rb.Plugin.__init__ (self)
@@ -250,19 +256,25 @@ class ArtDisplayPlugin (rb.Plugin):
 		sp = shell.get_player ()
 		self.pec_id = sp.connect ('playing-song-changed', self.playing_entry_changed)
 		self.pc_id = sp.connect ('playing-changed', self.playing_changed)
-		self.art_widget = FadingImage (self.find_file (ART_MISSING_ICON + ".svg"))
-		self.art_widget.set_padding (0, 5)
+		db = shell.get_property ("db")
+		self.eemr_art_id = db.connect_after ('entry-extra-metadata-request::rb:coverArt', self.cover_art_request)
+		self.eemn_art_id = db.connect_after ('entry-extra-metadata-notify::rb:coverArt', self.cover_art_notify)
+		self.eemn_art_uri_id = db.connect_after ('entry-extra-metadata-notify::rb:coverArt-uri', self.cover_art_uri_notify)
+		self.art_widget = ArtDisplayWidget (self.find_file (ART_MISSING_ICON + ".svg"))
 		shell.add_widget (self.art_widget, rb.SHELL_UI_LOCATION_SIDEBAR)
 		self.art_db = CoverArtDatabase ()
-		self.current_entry = None
-		entry = sp.get_playing_entry ()
-		self.playing_entry_changed (sp, entry)
+		self.current_entry, self.current_pixbuf = None, None
+		self.playing_entry_changed (sp, sp.get_playing_entry ())
 	
 	def deactivate (self, shell):
 		self.shell = None
 		sp = shell.get_player ()
 		sp.disconnect (self.pec_id)
 		sp.disconnect (self.pc_id)
+		db = shell.get_property ("db")
+		db.disconnect (self.eemr_art_id)
+		db.disconnect (self.eemn_art_id)
+		db.disconnect (self.eemn_art_uri_id)
 		shell.remove_widget (self.art_widget, rb.SHELL_UI_LOCATION_SIDEBAR)
 		self.art_widget.disconnect_handlers ()
 		self.art_widget = None
@@ -275,16 +287,60 @@ class ArtDisplayPlugin (rb.Plugin):
 		self.set_entry(entry)
 
 	def set_entry (self, entry):
-		if entry != self.current_entry and entry is not None:
-			db = self.shell.get_property ("db")
-
-			self.art_widget.set_current_art (None, True)
-			self.art_widget.show ()
-			# Intitates search in the database (which checks art cache, internet etc.)
-			self.current_entry = entry
-			self.art_db.get_pixbuf(db, entry, self.on_get_pixbuf_completed)
-
-	def on_get_pixbuf_completed(self, entry, pixbuf):
-		# Set the pixbuf for the entry returned from the art db
 		if entry == self.current_entry:
-			self.art_widget.set_current_art (pixbuf, False)
+			return
+		db = self.shell.get_property ("db")
+
+		self.art_widget.set (entry, None, None, True)
+		self.art_widget.show ()
+		# Intitates search in the database (which checks art cache, internet etc.)
+		self.current_entry = entry
+		self.current_pixbuf = None
+		self.art_db.get_pixbuf(db, entry, self.on_get_pixbuf_completed)
+
+	def on_get_pixbuf_completed(self, entry, pixbuf, uri):
+		# Set the pixbuf for the entry returned from the art db
+		if entry != self.current_entry:
+			return
+		self.current_pixbuf = pixbuf
+		self.art_widget.set (entry, pixbuf, uri, False)
+		if pixbuf:
+			db = self.shell.get_property ("db")
+			# This might be from a playing-changed signal, 
+			# in which case consumers won't be ready yet.
+			def idle_emit_art():
+				db.emit_entry_extra_metadata_notify (entry, "rb:coverArt", pixbuf)
+				return False
+			gobject.idle_add(idle_emit_art)
+
+	def cover_art_request (self, db, entry):
+		if entry == self.current_entry:
+			return self.current_pixbuf
+
+	def cover_art_notify (self, db, entry, field, metadata):
+		if entry != self.current_entry:
+			return
+		if not isinstance (metadata, gtk.gdk.Pixbuf):
+			return
+		self.art_db.cancel_get_pixbuf (entry)
+		if self.current_pixbuf == metadata:
+			return
+		self.art_widget.set (entry, metadata, None, False)
+
+	def cover_art_uri_notify (self, db, entry, field, metadata):
+		if entry != self.current_entry or not metadata:
+			return
+		uri = str (metadata)
+		def loader_cb (data):
+			if data and len (data) >= 1000:
+				pbl = gtk.gdk.PixbufLoader ()
+				try:
+					if pbl.write (data) and pbl.close ():
+						pixbuf = pbl.get_pixbuf ()
+						if pixbuf:
+							self.art_db.cancel_get_pixbuf (entry)
+							self.on_get_pixbuf_completed (entry, pixbuf, uri)
+				except GError:
+					pass
+		rb.Loader().get_url (uri, loader_cb)
+
