@@ -64,8 +64,6 @@
 #include "rb-file-helpers.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
-#include "rb-station-properties-dialog.h"
-#include "rb-new-station-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-shell-player.h"
@@ -148,7 +146,7 @@ GType rb_lastfm_src_get_type (void);
 static const char* const radio_options[][3] = {
 	{N_("Similar Artists radio"), "lastfm://artist/%s/similarartists", N_("Artists similar to %s")},
 	{N_("Tag radio"), "lastfm://globaltags/%s", N_("Tracks tagged with %s")},
-	{N_("Artist Fan radio"), "lastfm://artist/%s/fans", N_("Artists like by fans of %s")},
+	{N_("Artist Fan radio"), "lastfm://artist/%s/fans", N_("Artists liked by fans of %s")},
 	{N_("Group radio"), "lastfm://group/%s", N_("Tracks liked by the %s group")},
 	{NULL, NULL, NULL}
 };
@@ -273,24 +271,11 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 static void
 rb_lastfm_source_init (RBLastfmSource *source)
 {
-	gint size;
-	GdkPixbuf *pixbuf;
-
 	source->priv = RB_LASTFM_SOURCE_GET_PRIVATE (source);
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
-
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					   "stock_channel",
-					   size,
-					   0, NULL);
-	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
-	if (pixbuf != NULL) {
-		g_object_unref (pixbuf);
-	}
 }
 
 static void
@@ -1048,24 +1033,32 @@ rb_lastfm_source_title_from_uri (char *uri)
 		 *  will handle for us.
 		 */
 		if (data[4] != NULL) {
-			if (strcmp (data[4], "similarartists") == 0)
+			if (strcmp (data[4], "similarartists") == 0) {
 				title = g_strdup_printf (_("Artists similar to %s"), data[3]);
-			if (strcmp (data[4], "fans") == 0)
+			} if (strcmp (data[4], "fans") == 0) {
 				title = g_strdup_printf (_("Artists liked by fans of %s"), data[3]);
+			}
 		}
 
 	}
 
 	if (title == NULL && strcmp (data[2], "user") == 0) {
-		if (strcmp(data[4], "neighbours") == 0)
+		if (strcmp (data[4], "neighbours") == 0) {
 			title = g_strdup_printf (_("%s's Neighbour Radio"), data[3]);
-		if (strcmp(data[4], "recommended") == 0)
+		} else if (strcmp (data[4], "recommended") == 0) {
 			title = g_strdup_printf (_("%s's Recommended Radio: %s percent"), data[3], data[5]);
+		} else if (strcmp (data[4], "personal") == 0) {
+			title = g_strdup_printf (_("%s's Personal Radio"), data[3]);
+		}
 		/* subscriber? */
 	}
 
+	if (title == NULL && strcmp (data[2], "usertags") == 0) {
+		title = g_strdup_printf (_("%s's %s Radio"), data[3], data[4]);
+	}
+
 	if (title == NULL) {
-		title = g_strstrip (uri);
+		title = g_strdup (g_strstrip (uri));
 	}
 
 	g_strfreev (data);
