@@ -1,7 +1,7 @@
 /*
  * heavily based on code from Gedit
  *
- * Copyright (C) 2002-2005 - Paolo Maggi 
+ * Copyright (C) 2002-2005 - Paolo Maggi
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, 
- * Boston, MA 02110-1301  USA. 
+ * Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #ifndef __RB_PLUGIN_H__
@@ -47,6 +47,10 @@ typedef struct
 	GObject parent;
 } RBPlugin;
 
+typedef void		(*RBPluginActivationFunc)	(RBPlugin *plugin, RBShell *shell);
+typedef GtkWidget *	(*RBPluginWidgetFunc)		(RBPlugin *plugin);
+typedef gboolean	(*RBPluginBooleanFunc)		(RBPlugin *plugin);
+
 /*
  * Class definition
  */
@@ -55,19 +59,14 @@ typedef struct
 	GObjectClass parent_class;
 
 	/* Virtual public methods */
-	
-	void 		(*activate)		(RBPlugin *plugin,
-						 RBShell *shell);
-	void 		(*deactivate)		(RBPlugin *plugin,
-						 RBShell *shell);
 
-	GtkWidget*	(*create_configure_dialog)
-						(RBPlugin *plugin);
+	RBPluginActivationFunc		activate;
+	RBPluginActivationFunc		deactivate;
+	RBPluginWidgetFunc		create_configure_dialog;
 
 	/* Plugins should not override this, it's handled automatically by
 	   the RbPluginClass */
-	gboolean 	(*is_configurable)
-						(RBPlugin *plugin);
+	RBPluginBooleanFunc		is_configurable;
 } RBPluginClass;
 
 /*
@@ -79,11 +78,16 @@ void 		 rb_plugin_activate		(RBPlugin *plugin,
 						 RBShell *shell);
 void 		 rb_plugin_deactivate		(RBPlugin *plugin,
 						 RBShell *shell);
-				 
+
 gboolean	 rb_plugin_is_configurable	(RBPlugin *plugin);
-GtkWidget	*rb_plugin_create_configure_dialog		
+GtkWidget	*rb_plugin_create_configure_dialog
 						(RBPlugin *plugin);
 
+char *		 rb_plugin_find_file		(RBPlugin *plugin,
+						 const char *file);
+
+GList *          rb_get_plugin_paths            (void);
+
 /*
  * Utility macro used to register plugins
  *
@@ -141,7 +145,6 @@ register_rb_plugin (GTypeModule *module)					\
 	return plugin_name##_type;						\
 }
 
-
 #define RB_PLUGIN_REGISTER_TYPE(type_name)                                      \
         type_name##_register_type (plugin_module_type)
 
@@ -195,8 +198,6 @@ type_name##_register_type (GTypeModule *module) 				\
 	return type_name##_type_id;						\
 }
 
-
 G_END_DECLS
 
 #endif  /* __GEDIT_PLUGIN_H__ */
-
