@@ -80,21 +80,21 @@ static gboolean rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source
 						   GtkSelectionData *selection_data);
 static gboolean rb_sourcelist_model_row_draggable (RbTreeDragSource *drag_source,
 						   GList *path_list);
-static void rb_sourcelist_model_row_inserted_cb (GtkTreeModel *model, 
-						 GtkTreePath *path, 
-						 GtkTreeIter *iter, 
+static void rb_sourcelist_model_row_inserted_cb (GtkTreeModel *model,
+						 GtkTreePath *path,
+						 GtkTreeIter *iter,
 						 RBSourceListModel *sourcelist);
 static void rb_sourcelist_model_row_deleted_cb (GtkTreeModel *model,
 						GtkTreePath *path,
 						RBSourceListModel *sourcelist);
 
-
 static guint rb_sourcelist_model_signals[LAST_SIGNAL] = { 0 };
 
 enum {
 	TARGET_PROPERTY,
 	TARGET_SOURCE,
 	TARGET_URIS,
+	TARGET_ENTRIES,
 	TARGET_DELETE
 };
 
@@ -102,21 +102,21 @@ static const GtkTargetEntry sourcelist_targets[] = { { "text/x-rhythmbox-album",
 						     { "text/x-rhythmbox-artist", 0, TARGET_PROPERTY },
 						     { "text/x-rhythmbox-genre", 0, TARGET_PROPERTY },
 						     { "application/x-rhythmbox-source", 0, TARGET_SOURCE },
+						     { "application/x-rhythmbox-entry", 0, TARGET_ENTRIES },
 						     { "text/uri-list", 0, TARGET_URIS },
 						     { "application/x-delete-me", 0, TARGET_DELETE }};
 
 static GtkTargetList *sourcelist_drag_target_list = NULL;
 
-G_DEFINE_TYPE_EXTENDED (RBSourceListModel, 
-                        rb_sourcelist_model, 
+G_DEFINE_TYPE_EXTENDED (RBSourceListModel,
+                        rb_sourcelist_model,
                         GTK_TYPE_TREE_MODEL_FILTER,
-                        0, 
-                        G_IMPLEMENT_INTERFACE (RB_TYPE_TREE_DRAG_SOURCE, 
+                        0,
+                        G_IMPLEMENT_INTERFACE (RB_TYPE_TREE_DRAG_SOURCE,
                                                rb_sourcelist_model_drag_source_init)
-                        G_IMPLEMENT_INTERFACE (RB_TYPE_TREE_DRAG_DEST, 
+                        G_IMPLEMENT_INTERFACE (RB_TYPE_TREE_DRAG_DEST,
                                                rb_sourcelist_model_drag_dest_init));
 
-
 static void
 rb_sourcelist_model_class_init (RBSourceListModelClass *class)
 {
@@ -134,13 +134,13 @@ rb_sourcelist_model_class_init (RBSourceListModelClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBSourceListModelClass, drop_received),
 			      NULL, NULL,
-			      rb_marshal_VOID__POINTER_INT_POINTER,
+			      rb_marshal_VOID__OBJECT_INT_POINTER,
 			      G_TYPE_NONE,
 			      3,
-			      G_TYPE_POINTER, G_TYPE_INT, G_TYPE_POINTER);
+			      RB_TYPE_SOURCE, G_TYPE_INT, G_TYPE_POINTER);
 
 	if (!sourcelist_drag_target_list)
-		sourcelist_drag_target_list = 
+		sourcelist_drag_target_list =
 			gtk_target_list_new (sourcelist_targets,
 					     G_N_ELEMENTS (sourcelist_targets));
 
@@ -150,18 +150,18 @@ rb_sourcelist_model_class_init (RBSourceListModelClass *class)
 static void
 rb_sourcelist_model_drag_dest_init (RbTreeDragDestIface *iface)
 {
-  iface->drag_data_received = rb_sourcelist_model_drag_data_received;
-  iface->row_drop_possible = rb_sourcelist_model_row_drop_possible;
-  iface->row_drop_position = rb_sourcelist_model_row_drop_position;
-  iface->get_drag_target = rb_sourcelist_model_get_drag_target;
+  iface->rb_drag_data_received = rb_sourcelist_model_drag_data_received;
+  iface->rb_row_drop_possible = rb_sourcelist_model_row_drop_possible;
+  iface->rb_row_drop_position = rb_sourcelist_model_row_drop_position;
+  iface->rb_get_drag_target = rb_sourcelist_model_get_drag_target;
 }
 
 static void
 rb_sourcelist_model_drag_source_init (RbTreeDragSourceIface *iface)
 {
-  iface->row_draggable = rb_sourcelist_model_row_draggable;
-  iface->drag_data_get = rb_sourcelist_model_drag_data_get;
-  iface->drag_data_delete = rb_sourcelist_model_drag_data_delete;
+  iface->rb_row_draggable = rb_sourcelist_model_row_draggable;
+  iface->rb_drag_data_get = rb_sourcelist_model_drag_data_get;
+  iface->rb_drag_data_delete = rb_sourcelist_model_drag_data_delete;
 }
 
 void
@@ -181,7 +181,7 @@ rb_sourcelist_model_set_dnd_targets (RBSourceListModel *sourcelist,
 					     sourcelist_targets, n_targets,
 					     GDK_ACTION_COPY);
 }
-		
+
 static void
 rb_sourcelist_model_init (RBSourceListModel *model)
 {
@@ -215,24 +215,25 @@ rb_sourcelist_model_new (void)
 	column_types[RB_SOURCELIST_MODEL_COLUMN_PLAYING] = G_TYPE_BOOLEAN;
 	column_types[RB_SOURCELIST_MODEL_COLUMN_PIXBUF] = GDK_TYPE_PIXBUF;
 	column_types[RB_SOURCELIST_MODEL_COLUMN_NAME] = G_TYPE_STRING;
-	column_types[RB_SOURCELIST_MODEL_COLUMN_SOURCE] = G_TYPE_POINTER;
+	column_types[RB_SOURCELIST_MODEL_COLUMN_SOURCE] = G_TYPE_OBJECT;
 	column_types[RB_SOURCELIST_MODEL_COLUMN_ATTRIBUTES] = PANGO_TYPE_ATTR_LIST;
 	column_types[RB_SOURCELIST_MODEL_COLUMN_VISIBILITY] = G_TYPE_BOOLEAN;
-	store = gtk_tree_store_newv (RB_SOURCELIST_MODEL_N_COLUMNS, 
+	store = gtk_tree_store_newv (RB_SOURCELIST_MODEL_N_COLUMNS,
 				     column_types);
 
- 	model = RB_SOURCELIST_MODEL (g_object_new (RB_TYPE_SOURCELIST_MODEL, 
+ 	model = RB_SOURCELIST_MODEL (g_object_new (RB_TYPE_SOURCELIST_MODEL,
 						   "child-model", store,
 						   "virtual-root", NULL,
 						   NULL));
-	
+	g_object_unref (store);
+
 	/* create marker rows used to separate source groups */
 	for (i = 0; i < RB_SOURCELIST_GROUP_LAST; i++) {
 		GtkTreeIter iter;
 		GtkTreePath *path;
 
 		gtk_tree_store_append (store, &iter, NULL);
-		gtk_tree_store_set (store, &iter, 
+		gtk_tree_store_set (store, &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_NAME, "",
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, NULL,
 				    RB_SOURCELIST_MODEL_COLUMN_VISIBILITY, FALSE,
@@ -243,16 +244,16 @@ rb_sourcelist_model_new (void)
 		gtk_tree_path_free (path);
 	}
 
-	/* ensure the group markers get updated as sources are added and removed */ 
+	/* ensure the group markers get updated as sources are added and removed */
 	g_signal_connect_object (G_OBJECT (store), "row-inserted",
-				 G_CALLBACK (rb_sourcelist_model_row_inserted_cb), 
+				 G_CALLBACK (rb_sourcelist_model_row_inserted_cb),
 				 model, 0);
 	g_signal_connect_object (G_OBJECT (store), "row-deleted",
-				 G_CALLBACK (rb_sourcelist_model_row_deleted_cb), 
+				 G_CALLBACK (rb_sourcelist_model_row_deleted_cb),
 				 model, 0);
 
-	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (model), 
-						(GtkTreeModelFilterVisibleFunc) rb_sourcelist_model_is_row_visible, 
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (model),
+						(GtkTreeModelFilterVisibleFunc) rb_sourcelist_model_is_row_visible,
 						model, NULL);
 
 	g_free (column_types);
@@ -311,10 +312,12 @@ rb_sourcelist_model_is_row_visible (GtkTreeModel *model,
 
 	gtk_tree_model_get (GTK_TREE_MODEL (model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
-	
+
 	if (source != NULL) {
-		gboolean visible;	
-		g_object_get (G_OBJECT (source), "visibility", &visible, NULL);
+		gboolean visible;
+		g_object_get (source, "visibility", &visible, NULL);
+
+		g_object_unref (source);
 
 		return visible;
 	} else {
@@ -328,21 +331,22 @@ rb_sourcelist_model_row_is_separator (GtkTreeModel *model,
 				      RBSourceListModel *sourcelist)
 {
 	GtkTreeIter real_iter;
-	RBSource *source; 
+	RBSource *source;
 
 	/* rows with actual sources are never separators */
 	gtk_tree_model_get (GTK_TREE_MODEL (model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
-	if (source != NULL)
+	if (source != NULL) {
+		g_object_unref (source);
 		return FALSE;
+	}
 
-	gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (sourcelist), 
-							  &real_iter, 
+	gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (sourcelist),
+							  &real_iter,
 							  iter);
 	return real_row_is_separator (sourcelist, &real_iter);
 }
 
-
 static int
 get_group_for_path (RBSourceListModel *model, GtkTreePath *path)
 {
@@ -374,20 +378,25 @@ rb_sourcelist_model_drag_data_received (RbTreeDragDest *drag_dest,
 
 	g_return_val_if_fail (RB_IS_SOURCELIST_MODEL (drag_dest), FALSE);
 	model = RB_SOURCELIST_MODEL (drag_dest);
-	
-	if (selection_data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
+
+	if (selection_data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    selection_data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
 		GtkTreeIter iter;
-		RBSource *target;
-		rb_debug ("text/uri-list drag data received");
-		
-		if (dest == NULL || !gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, dest))
-			target = NULL;
-		else
+		RBSource *target = NULL;
+
+		rb_debug ("text/uri-list or application/x-rhythmbox-entry drag data received");
+
+		if (dest != NULL && gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, dest)) {
 			gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 					    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		
+		}
+
 		g_signal_emit (G_OBJECT (model), rb_sourcelist_model_signals[DROP_RECEIVED],
 			       0, target, pos, selection_data);
+
+		if (target != NULL)
+			g_object_unref (target);
+
 		return TRUE;
 	}
 
@@ -416,23 +425,24 @@ rb_sourcelist_model_drag_data_received (RbTreeDragDest *drag_dest,
 			return FALSE;
 
 		real_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-		
+
 		path_str = g_strndup ((char *) selection_data->data, selection_data->length);
-	
+
 		path = gtk_tree_path_new_from_string (path_str);
 		gtk_tree_model_get_iter (GTK_TREE_MODEL (model),
 					 &iter, path);
 		gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model),
 								  &real_iter, &iter);
 
-		real_dest = 
-			gtk_tree_model_filter_convert_path_to_child_path (GTK_TREE_MODEL_FILTER (model), 
+		real_dest =
+			gtk_tree_model_filter_convert_path_to_child_path (GTK_TREE_MODEL_FILTER (model),
 									  dest);
 
 		gtk_tree_model_get (GTK_TREE_MODEL (real_model), &real_iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, 
+				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
 				    -1);
-		g_object_get (G_OBJECT (source), "sourcelist-group", &group, NULL);
+		g_object_get (source, "sourcelist-group", &group, NULL);
+		g_object_unref (source);
 
 		/* restrict sources to within their group */
 		dest_group = get_group_for_path (model, real_dest);
@@ -481,13 +491,9 @@ rb_sourcelist_model_row_drop_possible (RbTreeDragDest *drag_dest,
 	g_return_val_if_fail (RB_IS_SOURCELIST_MODEL (drag_dest), FALSE);
 	model = RB_SOURCELIST_MODEL (drag_dest);
 
-	if (selection_data->type == gdk_atom_intern ("text/uri-list", TRUE)
-	    && !dest)
-		return FALSE;
-
 	if (!dest)
 		return TRUE;
-			
+
 	/* Call the superclass method */
 	return gtk_tree_drag_dest_row_drop_possible (GTK_TREE_DRAG_DEST (GTK_TREE_STORE (model)),
 						     dest, selection_data);
@@ -498,17 +504,23 @@ path_is_droppable (RBSourceListModel *model,
 		   GtkTreePath *dest)
 {
 	GtkTreeIter iter;
-	
+	gboolean res;
+
+	res = FALSE;
+
 	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, dest)) {
 		RBSource *source;
-		
+
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
-		
-		if (source)
-			return rb_source_can_paste (source);
+
+		if (source != NULL) {
+			res = rb_source_can_paste (source);
+			g_object_unref (source);
+		}
 	}
-	return FALSE;
+
+	return res;
 }
 
 static gboolean
@@ -516,20 +528,25 @@ path_is_reorderable (RBSourceListModel *model,
 		     GtkTreePath *dest)
 {
 	GtkTreeIter iter;
+
 	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, dest)) {
 		RBSource *source;
 		RBSourceListGroup group;
+
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
+
 		if (source == NULL)
 			return FALSE;
 
-		g_object_get (G_OBJECT (source), "sourcelist-group", &group, NULL);
+		g_object_get (source, "sourcelist-group", &group, NULL);
+		g_object_unref (source);
 
 		/* fixed and transient sources are not reorderable, everything else is */
 		return (group != RB_SOURCELIST_GROUP_FIXED &&
 			group != RB_SOURCELIST_GROUP_TRANSIENT);
 	}
+
 	return FALSE;
 }
 
@@ -551,11 +568,11 @@ rb_sourcelist_model_row_drop_position (RbTreeDragDest   *drag_dest,
 				*pos = GTK_TREE_VIEW_DROP_AFTER;
 				ret = TRUE;
 			}
-			
+
 			gtk_tree_path_free (test_path);
 			return ret;
 		}
-		
+
 		if (*pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
 			*pos = GTK_TREE_VIEW_DROP_BEFORE;
 		else if (*pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
@@ -564,8 +581,9 @@ rb_sourcelist_model_row_drop_position (RbTreeDragDest   *drag_dest,
 		return TRUE;
 	}
 
-	if (g_list_find (targets, gdk_atom_intern ("text/uri-list", TRUE))) {
-		rb_debug ("text/uri-list type");
+	if (g_list_find (targets, gdk_atom_intern ("text/uri-list", TRUE)) ||
+	    g_list_find (targets, gdk_atom_intern ("application/x-rhythmbox-entry", TRUE))) {
+		rb_debug ("text/uri-list or application/x-rhythmbox-entry type");
 		if (dest_path && !path_is_droppable (RB_SOURCELIST_MODEL (model), dest_path))
 			return FALSE;
 
@@ -596,9 +614,15 @@ rb_sourcelist_model_get_drag_target (RbTreeDragDest *drag_dest,
 		/* always accept rb source path if offered */
 		return gdk_atom_intern ("application/x-rhythmbox-source", TRUE);
 	}
-	
+
 	if (path) {
-		/* only accept text/uri-list drops into existing sources */
+		/* only accept text/uri-list or application/x-rhythmbox-entry drops into existing sources */
+		GdkAtom entry_atom;
+
+		entry_atom = gdk_atom_intern ("application/x-rhythmbox-entry", FALSE);
+		if (g_list_find (context->targets, entry_atom))
+			return entry_atom;
+
 		return gdk_atom_intern ("text/uri-list", FALSE);
 	}
 
@@ -606,8 +630,6 @@ rb_sourcelist_model_get_drag_target (RbTreeDragDest *drag_dest,
 					  target_list);
 }
 
-
-
 static gboolean
 rb_sourcelist_model_row_draggable (RbTreeDragSource *drag_source,
 				   GList *path_list)
@@ -615,25 +637,26 @@ rb_sourcelist_model_row_draggable (RbTreeDragSource *drag_source,
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	GtkTreeModel *model = GTK_TREE_MODEL (drag_source);
-	
+
 	/* we don't support multi selection */
 	g_return_val_if_fail (g_list_length (path_list) == 1, FALSE);
 
 	path = gtk_tree_row_reference_get_path (path_list->data);
 
-	if (gtk_tree_model_get_iter (model, &iter, path)) {
+	if (path && gtk_tree_model_get_iter (model, &iter, path)) {
 		RBSource *source;
 		RBSourceListGroup group;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
-	
-		g_object_get (G_OBJECT (source), "sourcelist-group", &group, NULL);
+
+		g_object_get (source, "sourcelist-group", &group, NULL);
+		g_object_unref (source);
 
 		return (group != RB_SOURCELIST_GROUP_FIXED &&
 			group != RB_SOURCELIST_GROUP_TRANSIENT);
 	}
-		
+
 	return FALSE;
 }
 
@@ -647,13 +670,17 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 	guint target;
 
 	path = gtk_tree_row_reference_get_path (path_list->data);
+	if (path == NULL)
+		return FALSE;
+
 	if (!gtk_target_list_find (sourcelist_drag_target_list,
 				   selection_data->target,
 				   &target)) {
 		return FALSE;
 	}
 
-	if (target == TARGET_SOURCE) {
+	switch (target) {
+	case TARGET_SOURCE:
 		rb_debug ("getting drag data as rb source path");
 		path_str = gtk_tree_path_to_string (path);
 		gtk_selection_data_set (selection_data,
@@ -663,7 +690,9 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 		g_free (path_str);
 		gtk_tree_path_free (path);
 		return TRUE;
-	} else if (target == TARGET_URIS) {
+	case TARGET_URIS:
+	case TARGET_ENTRIES:
+	{
 		RBSource *source;
 		RhythmDBQueryModel *query_model;
 		GtkTreeIter iter;
@@ -673,30 +702,41 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 		rb_debug ("getting drag data as uri list");
 		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_source), &iter, path))
 			return FALSE;
-			
+
 		data = g_string_new ("");
 		gtk_tree_model_get (GTK_TREE_MODEL (drag_source), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
-		g_object_get (G_OBJECT (source), "query-model", &query_model, NULL);
+		g_object_get (source, "query-model", &query_model, NULL);
+		g_object_unref (source);
 
 		if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (query_model), &iter)) {
-			g_object_unref (G_OBJECT (query_model));
+			g_object_unref (query_model);
 			return FALSE;
 		}
 
 		do {
 			RhythmDBEntry *entry;
+
 			if (first) {
 				g_string_append(data, "\r\n");
 				first = FALSE;
 			}
 
 			entry = rhythmdb_query_model_iter_to_entry (query_model, &iter);
-			g_string_append (data, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+			if (target == TARGET_URIS) {
+				g_string_append (data, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+			} else {
+				g_string_append_printf (data,
+							"%lu",
+							rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_ENTRY_ID));
+			}
+
+			rhythmdb_entry_unref (entry);
 
 		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (query_model), &iter));
-		g_object_unref (G_OBJECT (query_model));
-		
+
+		g_object_unref (query_model);
+
 		gtk_selection_data_set (selection_data,
 					selection_data->target,
 					8, (guchar *) data->str,
@@ -704,8 +744,8 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 
 		g_string_free (data, TRUE);
 		return TRUE;
-
-	} else {
+	}
+	default:
 		/* unsupported target */
 		return FALSE;
 	}
@@ -718,16 +758,16 @@ rb_sourcelist_model_drag_data_delete (RbTreeDragSource *drag_source,
 	return TRUE;
 }
 
-static void 
-rb_sourcelist_model_row_inserted_cb (GtkTreeModel *model, 
-				     GtkTreePath *path, 
-				     GtkTreeIter *iter, 
+static void
+rb_sourcelist_model_row_inserted_cb (GtkTreeModel *model,
+				     GtkTreePath *path,
+				     GtkTreeIter *iter,
 				     RBSourceListModel *sourcelist)
 {
 	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sourcelist));
 }
 
-static void 
+static void
 rb_sourcelist_model_row_deleted_cb (GtkTreeModel *model,
 			            GtkTreePath *path,
 				    RBSourceListModel *sourcelist)
@@ -736,9 +776,8 @@ rb_sourcelist_model_row_deleted_cb (GtkTreeModel *model,
 }
 
 GtkTreePath *
-rb_sourcelist_model_get_group_path (RBSourceListModel *sourcelist, 
+rb_sourcelist_model_get_group_path (RBSourceListModel *sourcelist,
 				    RBSourceListGroup group)
 {
 	return gtk_tree_row_reference_get_path (sourcelist->priv->groups[group]);
 }
-
