@@ -86,10 +86,10 @@ static void rb_sourcelist_get_property (GObject *object,
 					guint prop_id,
 					GValue *value,
 					GParamSpec *pspec);
-static gboolean rb_sourcelist_source_to_iter (RBSourceList *sourcelist, 
+static gboolean rb_sourcelist_source_to_iter (RBSourceList *sourcelist,
 					      RBSource *source,
 					      GtkTreeIter *iter);
-static gboolean rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist, 
+static gboolean rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist,
 						      RBSource *source,
 						      GtkTreeIter *iter);
 static void rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
@@ -106,9 +106,8 @@ static gboolean key_release_cb (GtkTreeView *treeview,
 				RBSourceList *sourcelist);
 static gboolean popup_menu_cb (GtkTreeView *treeview, RBSourceList *sourcelist);
 static void name_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data);
-static void visibility_notify_cb (GObject *obj, GParamSpec *pspec, 
+static void visibility_notify_cb (GObject *obj, GParamSpec *pspec,
 				  gpointer data);
-static void icon_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data);
 static void source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 				   const char *text, RBSourceList *sourcelist);
 static void rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
@@ -118,8 +117,7 @@ static void rb_sourcelist_update_expander_visibility (RBSourceList *sourcelist);
 
 static guint rb_sourcelist_signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (RBSourceList, rb_sourcelist, GTK_TYPE_SCROLLED_WINDOW) 
-
+G_DEFINE_TYPE (RBSourceList, rb_sourcelist, GTK_TYPE_SCROLLED_WINDOW)
 
 static void
 rb_sourcelist_class_init (RBSourceListClass *class)
@@ -191,7 +189,7 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 			      rb_marshal_BOOLEAN__POINTER,
 			      G_TYPE_BOOLEAN,
 			      1,
-			      G_TYPE_POINTER);
+			      RB_TYPE_SOURCE);
 
 	g_type_class_add_private (class, sizeof (RBSourceListPrivate));
 }
@@ -282,6 +280,12 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 static void
 rb_sourcelist_finalize (GObject *object)
 {
+	RBSourceList *sourcelist = RB_SOURCELIST (object);
+
+	if (sourcelist->priv->filter_model != NULL) {
+		g_object_unref (sourcelist->priv->filter_model);
+	}
+
 	G_OBJECT_CLASS (rb_sourcelist_parent_class)->finalize (object);
 }
 
@@ -303,7 +307,7 @@ rb_sourcelist_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_sourcelist_get_property (GObject *object,
 			    guint prop_id,
 			    GValue *value,
@@ -324,7 +328,6 @@ rb_sourcelist_get_property (GObject *object,
 	}
 }
 
-
 GtkWidget *
 rb_sourcelist_new (RBShell *shell)
 {
@@ -345,23 +348,45 @@ rb_sourcelist_new (RBShell *shell)
 	return GTK_WIDGET (sourcelist);
 }
 
+static void
+icon_notify_cb (RBSource *source,
+		GParamSpec *pspec,
+		RBSourceList *sourcelist)
+{
+	GtkTreeIter iter;
+
+	if (rb_sourcelist_source_to_iter (sourcelist, source, &iter)) {
+		GdkPixbuf *pixbuf;
+
+		g_object_get (source, "icon", &pixbuf, NULL);
+		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model),
+				    &iter,
+				    RB_SOURCELIST_MODEL_COLUMN_PIXBUF, pixbuf, -1);
+		if (pixbuf != NULL) {
+			g_object_unref (pixbuf);
+		}
+	}
+
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
+}
+
 void
 rb_sourcelist_append (RBSourceList *sourcelist,
 		      RBSource *source,
 		      RBSource *parent)
 {
 	GtkTreeIter iter;
-	PangoAttrList *attrs = pango_attr_list_new ();
-	const char *name;
+	PangoAttrList *attrs;
+	char *name;
 	GdkPixbuf *pixbuf;
 	gboolean visible;
 
 	g_return_if_fail (RB_IS_SOURCELIST (sourcelist));
 	g_return_if_fail (RB_IS_SOURCE (source));
 
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
-	g_object_get (G_OBJECT (source), "icon", &pixbuf, NULL);
-	g_object_get (G_OBJECT (source), "visibility", &visible, NULL);
+	g_object_get (source, "name", &name, NULL);
+	g_object_get (source, "icon", &pixbuf, NULL);
+	g_object_get (source, "visibility", &visible, NULL);
 
 	if (parent) {
 		GtkTreeIter parent_iter;
@@ -409,22 +434,28 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 				if (!gtk_tree_model_iter_next (sourcelist->priv->real_model, &group_iter))
 					break;
 
-				gtk_tree_model_get (sourcelist->priv->real_model, 
-						    &group_iter, 
+				gtk_tree_model_get (sourcelist->priv->real_model,
+						    &group_iter,
 						    RB_SOURCELIST_MODEL_COLUMN_NAME, &check_name,
 						    -1);
 
 			} while (check_name && (strlen (check_name)) > 0 && (g_utf8_collate (name, check_name) > 0));
+
 			g_free (check_name);
 
 		} else {
 			rb_debug ("inserting source %p in group %d", source, group);
 			gtk_tree_model_get_iter (sourcelist->priv->real_model, &group_iter, group_path);
 		}
-		gtk_tree_store_insert_before (GTK_TREE_STORE (sourcelist->priv->real_model), 
+
+		gtk_tree_store_insert_before (GTK_TREE_STORE (sourcelist->priv->real_model),
 					      &iter, NULL, &group_iter);
+
+		gtk_tree_path_free (group_path);
 	}
 
+	attrs = pango_attr_list_new ();
+
 	gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_PIXBUF, pixbuf,
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, name,
@@ -433,9 +464,12 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 			    RB_SOURCELIST_MODEL_COLUMN_VISIBILITY, visible,
 			    -1);
 
+	pango_attr_list_unref (attrs);
+
 	if (pixbuf != NULL) {
 		g_object_unref (pixbuf);
 	}
+	g_free (name);
 
 	g_signal_connect_object (G_OBJECT (source), "notify::name", G_CALLBACK (name_notify_cb), sourcelist, 0);
 	g_signal_connect_object (G_OBJECT (source), "notify::visibility", G_CALLBACK (visibility_notify_cb), sourcelist, 0);
@@ -450,36 +484,46 @@ typedef struct _SourcePath {
 } SourcePath;
 
 static gboolean
-match_source_to_iter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+match_source_to_iter (GtkTreeModel *model,
+		      GtkTreePath *path,
+		      GtkTreeIter *iter,
 		      SourcePath *sp)
 {
 	RBSource *target = NULL;
-	
+	gboolean  res;
+
 	gtk_tree_model_get (model, iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+
+	res = FALSE;
 	if (target == sp->source) {
 		sp->path = gtk_tree_path_copy (path);
-		return TRUE;
+		res = TRUE;
 	}
 
-	return FALSE;
+	if (target != NULL) {
+		g_object_unref (target);
+	}
+
+	return res;
 }
 
 static gboolean
-rb_sourcelist_source_to_iter (RBSourceList *sourcelist, RBSource *source,
+rb_sourcelist_source_to_iter (RBSourceList *sourcelist,
+			      RBSource *source,
 			      GtkTreeIter *iter)
 {
 	SourcePath *sp = g_new0 (SourcePath,1);
 	gboolean ret = FALSE;
-	
+
 	sp->source = source;
-	
+
 	gtk_tree_model_foreach (sourcelist->priv->real_model, (GtkTreeModelForeachFunc) match_source_to_iter, sp);
-	
+
 	if (sp->path) {
 		ret = gtk_tree_model_get_iter (sourcelist->priv->real_model, iter, sp->path);
+		gtk_tree_path_free (sp->path);
 	}
 
-	gtk_tree_path_free (sp->path);
 	g_free (sp);
 	sp = NULL;
 
@@ -487,21 +531,22 @@ rb_sourcelist_source_to_iter (RBSourceList *sourcelist, RBSource *source,
 }
 
 static gboolean
-rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist, RBSource *source,
+rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist,
+				      RBSource *source,
 				      GtkTreeIter *iter)
 {
 	SourcePath *sp = g_new0 (SourcePath,1);
 	gboolean ret = FALSE;
-	
+
 	sp->source = source;
-	
+
 	gtk_tree_model_foreach (sourcelist->priv->filter_model, (GtkTreeModelForeachFunc) match_source_to_iter, sp);
 
 	if (sp->path) {
 		ret = gtk_tree_model_get_iter (sourcelist->priv->filter_model, iter, sp->path);
+		gtk_tree_path_free (sp->path);
 	}
 
-	gtk_tree_path_free (sp->path);
 	g_free (sp);
 	sp = NULL;
 
@@ -509,28 +554,32 @@ rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist, RBSource *source
 }
 
 void
-rb_sourcelist_edit_source_name (RBSourceList *sourcelist, RBSource *source)
+rb_sourcelist_edit_source_name (RBSourceList *sourcelist,
+				RBSource *source)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	
+
 	g_assert (rb_sourcelist_visible_source_to_iter (sourcelist, source, &iter));
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (sourcelist->priv->filter_model),
 					&iter);
-	
+
 	/* Make cell editable just for the moment.
 	   We'll turn it off once editing is done. */
 	g_object_set (G_OBJECT (sourcelist->priv->title_renderer), "editable", TRUE, NULL);
-	
+
 	gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (sourcelist->priv->treeview),
 					  path, sourcelist->priv->main_column,
 					  sourcelist->priv->title_renderer,
 					  TRUE);
-	
+
 	gtk_tree_path_free (path);
 }
 
-static void set_source_playing (RBSourceList *sourcelist, RBSource *source, gboolean playing)
+static void
+set_source_playing (RBSourceList *sourcelist,
+		    RBSource *source,
+		    gboolean playing)
 {
 	GtkTreeIter iter;
 	g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
@@ -539,7 +588,8 @@ static void set_source_playing (RBSourceList *sourcelist, RBSource *source, gboo
 }
 
 void
-rb_sourcelist_set_playing_source (RBSourceList *sourcelist, RBSource *source)
+rb_sourcelist_set_playing_source (RBSourceList *sourcelist,
+				  RBSource *source)
 {
 	if (sourcelist->priv->playing_source)
 		set_source_playing (sourcelist, sourcelist->priv->playing_source, FALSE);
@@ -550,7 +600,8 @@ rb_sourcelist_set_playing_source (RBSourceList *sourcelist, RBSource *source)
 }
 
 void
-rb_sourcelist_remove (RBSourceList *sourcelist, RBSource *source)
+rb_sourcelist_remove (RBSourceList *sourcelist,
+		      RBSource *source)
 {
 	GtkTreeIter iter;
 
@@ -559,12 +610,15 @@ rb_sourcelist_remove (RBSourceList *sourcelist, RBSource *source)
 	if (gtk_tree_store_iter_depth (GTK_TREE_STORE (sourcelist->priv->real_model), &iter) > 0) {
 		gboolean visible;
 
-		g_object_get (G_OBJECT (source), "visibility", &visible, NULL);
+		g_object_get (source, "visibility", &visible, NULL);
 		if (visible)
 			sourcelist->priv->child_source_count--;
 		rb_sourcelist_update_expander_visibility (sourcelist);
 	}
 
+	if (source == sourcelist->priv->playing_source)
+		rb_sourcelist_set_playing_source (sourcelist, NULL);
+
 	gtk_tree_store_remove (GTK_TREE_STORE (sourcelist->priv->real_model), &iter);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (source),
 					      G_CALLBACK (name_notify_cb), sourcelist);
@@ -576,7 +630,8 @@ rb_sourcelist_remove (RBSourceList *sourcelist, RBSource *source)
 }
 
 void
-rb_sourcelist_select (RBSourceList *sourcelist, RBSource *source)
+rb_sourcelist_select (RBSourceList *sourcelist,
+		      RBSource *source)
 {
 	GtkTreeIter iter;
 
@@ -596,19 +651,25 @@ rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 	if (!gtk_tree_selection_get_selected (sourcelist->priv->selection,
 					      &cindy, &iter))
 		return;
-	
+
 	gtk_tree_model_get (cindy, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 	if (target == NULL)
 		return;
+
 	g_return_if_fail (RB_IS_SOURCE (target));
 	source = target;
 	g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SELECTED], 0, source);
+
+	g_object_unref (target);
 }
 
 static void
-drop_received_cb (RBSourceListModel *model, RBSource *target, GtkTreeViewDropPosition pos,
-		  GtkSelectionData *data, RBSourceList *sourcelist)
+drop_received_cb (RBSourceListModel *model,
+		  RBSource *target,
+		  GtkTreeViewDropPosition pos,
+		  GtkSelectionData *data,
+		  RBSourceList *sourcelist)
 {
 	rb_debug ("drop recieved");
 	/* Proxy the signal. */
@@ -629,8 +690,10 @@ row_activated_cb (GtkTreeView *treeview,
 	g_return_if_fail (gtk_tree_model_get_iter (model, &iter, path));
 	gtk_tree_model_get (model, &iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 
-	if (target)
+	if (target != NULL) {
 		g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SOURCE_ACTIVATED], 0, target);
+		g_object_unref (target);
+	}
 }
 
 static gboolean
@@ -649,10 +712,15 @@ emit_show_popup (GtkTreeView *treeview,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 	if (target == NULL)
 		return FALSE;
+
 	g_return_val_if_fail (RB_IS_SOURCE (target), FALSE);
 
 	g_signal_emit (G_OBJECT (sourcelist), rb_sourcelist_signals[SHOW_POPUP], 0, target, &ret);
 
+	if (target != NULL) {
+		g_object_unref (target);
+	}
+
 	return ret;
 }
 
@@ -663,7 +731,7 @@ button_press_cb (GtkTreeView *treeview,
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	
+
 	if (event->button != 3)
 		return FALSE;
 
@@ -671,9 +739,9 @@ button_press_cb (GtkTreeView *treeview,
 					   &path, NULL, NULL, NULL) == FALSE) {
 		/* pointer is over empty space */
 		GtkUIManager *uimanager;
-		g_object_get (G_OBJECT (sourcelist->priv->shell), "ui-manager", &uimanager, NULL);
+		g_object_get (sourcelist->priv->shell, "ui-manager", &uimanager, NULL);
 		rb_gtk_action_popup_menu (uimanager, "/SourceListPopup");
-		g_object_unref (G_OBJECT (uimanager));
+		g_object_unref (uimanager);
 		return TRUE;
 	}
 
@@ -690,6 +758,7 @@ key_release_cb (GtkTreeView *treeview,
 {
 	GtkTreeIter iter;
 	RBSource *target;
+	gboolean res;
 
 	/* F2 = rename playlist */
 	if (event->keyval != GDK_F2)
@@ -697,21 +766,22 @@ key_release_cb (GtkTreeView *treeview,
 
 	if (!gtk_tree_selection_get_selected (sourcelist->priv->selection, NULL, &iter))
 		return FALSE;
-	
+
 	gtk_tree_model_get (sourcelist->priv->filter_model, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 	if (target == NULL)
 		return FALSE;
 
-	g_return_val_if_fail (RB_IS_SOURCE (target), FALSE);
+	res = FALSE;
 	if (rb_source_can_rename (target)) {
 		rb_sourcelist_edit_source_name (sourcelist, target);
-		return TRUE;
+		res = TRUE;
 	}
 
-	return FALSE;
-}
+	g_object_unref (target);
 
+	return res;
+}
 
 static gboolean
 popup_menu_cb (GtkTreeView *treeview,
@@ -721,38 +791,50 @@ popup_menu_cb (GtkTreeView *treeview,
 }
 
 static void
-name_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
+name_notify_cb (GObject *obj,
+		GParamSpec *pspec,
+		gpointer data)
 {
 	RBSourceList *sourcelist = RB_SOURCELIST (data);
 	RBSource *source = RB_SOURCE (obj);
 	GtkTreeIter iter;
-	gchar *name;
-	
+	char *name;
+
 	if (rb_sourcelist_source_to_iter (sourcelist, source, &iter)) {
 		g_object_get (obj, "name", &name, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model),
 				    &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_NAME, name, -1);
+		g_free (name);
 	}
 
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
 }
-	
+
 static void
-visibility_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
+visibility_notify_cb (GObject *obj,
+		      GParamSpec *pspec,
+		      gpointer data)
 {
 	RBSourceList *sourcelist = RB_SOURCELIST (data);
 	RBSource *source = RB_SOURCE (obj);
 	GtkTreeIter iter;
 	gboolean old_visibility;
 	gboolean new_visibility;
+	char *name;
+
+	g_object_get (obj,
+		      "visibility", &new_visibility,
+		      "name", &name,
+		      NULL);
+	rb_debug ("Source visibility changed: %s", name);
+	g_free (name);
 
 	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sourcelist->priv->filter_model));
 
 	g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
 	if (gtk_tree_store_iter_depth (GTK_TREE_STORE (sourcelist->priv->real_model), &iter) > 0) {
 
-		g_object_get (obj, "visibility", &new_visibility, NULL);
 		gtk_tree_model_get (GTK_TREE_MODEL (sourcelist->priv->real_model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_VISIBILITY, &old_visibility,
 				    -1);
@@ -771,34 +853,15 @@ visibility_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
 }
 
 static void
-icon_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
-{
-	RBSourceList *sourcelist = RB_SOURCELIST (data);
-	RBSource *source = RB_SOURCE (obj);
-	GtkTreeIter iter;
-	GdkPixbuf *pixbuf;
-	
-	if (rb_sourcelist_source_to_iter (sourcelist, source, &iter)) {
-		g_object_get (obj, "icon", &pixbuf, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model),
-				    &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_PIXBUF, pixbuf, -1);
-		if (pixbuf != NULL) {
-			g_object_unref (pixbuf);
-		}
-	}
-
-	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
-}
-
-static void
-rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-				    GtkTreeModel *tree_model, GtkTreeIter *iter,
+rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column,
+				    GtkCellRenderer *renderer,
+				    GtkTreeModel *tree_model,
+				    GtkTreeIter *iter,
 				    RBSourceList *sourcelist)
 {
 	char *str;
 	gboolean playing;
-	
+
 	gtk_tree_model_get (GTK_TREE_MODEL (sourcelist->priv->filter_model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, &str,
 			    RB_SOURCELIST_MODEL_COLUMN_PLAYING, &playing,
@@ -812,8 +875,10 @@ rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *
 }
 
 static void
-source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
-		       const char *text, RBSourceList *sourcelist)
+source_name_edited_cb (GtkCellRendererText *renderer,
+		       const char *pathstr,
+		       const char *text,
+		       RBSourceList *sourcelist)
 {
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -821,9 +886,9 @@ source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 
 	if (text[0] == '\0')
 		return;
-	
-	path = gtk_tree_path_new_from_string (pathstr);	
-	
+
+	path = gtk_tree_path_new_from_string (pathstr);
+
 	g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (sourcelist->priv->filter_model),
 						   &iter, path));
 	gtk_tree_model_get (sourcelist->priv->filter_model,
@@ -838,6 +903,8 @@ source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 	g_object_set (G_OBJECT (renderer), "editable", FALSE, NULL);
 
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (sourcelist->priv->treeview));
+
+	g_object_unref (source);
 }
 
 static void
