@@ -15,13 +15,14 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
 #include <gtk/gtkdialog.h>
 #include "rb-entry-view.h"
 #include "rhythmdb.h"
+#include "rb-source.h"
 
 #ifndef __RB_SONG_INFO_H
 #define __RB_SONG_INFO_H
@@ -54,7 +55,9 @@ typedef struct
 
 GType      rb_song_info_get_type (void);
 
-GtkWidget *rb_song_info_new      (RBEntryView *view);
+GtkWidget *rb_song_info_new      (RBSource *source, RBEntryView *entry_view);
+
+guint      rb_song_info_append_page (RBSongInfo *info, const char *title, GtkWidget *page);
 
 G_END_DECLS
 
